#!/bin/sh
#
# Description: SleepTimer installer
# Stable work on: [Saturn (1186) Universal firmware r6 (3Dgui)]
# Version: 0.6beta-20170330
#

VERSION="06beta-20170330"
DIRSCRIPT=$(cd $(dirname $0) && pwd)
exec 3>&1 4>&2
exec >$DIRSCRIPT/sleepTimer_install.log 2>&1
echo "Start installer..."
echo "Version: $VERSION"
####################################
####################################
## Global variables initialization
##
debug=0									# Global debug is active or not
latency=30								# Latency timer shutdown, minutes
####################################
####################################
etc=/usr/local/etc
rc=$etc/rcS
RCscript=$etc/rc.init/S98sleepTimer		# Main RC-start SleepTimer script
daemon="/tmp/sleepD.sh"					# Daemon for read out of DvdPlayer in named PIPE
script="/tmp/sleepT.sh"					# Main check script of SleepTimer
DPlog="/tmp/DvdPlayer.out"				# Path of named PIPE for out of DvdPlayer
RClog="/tmp/PressKey.log"				# Path of timestamp log of last press key on RC
debugDLogPath="/tmp/sleepD.log"			# Debug log path for sleepD.sh
debugTLogPath="/tmp/sleepT.log"			# Debug log path for sleepT.sh
s98err="/tmp/S98_error.log"				# Run-time error log for main RC-start SleepTimer script
####################################
####################################
## Created main RC-start SleepTimer script
##
if [ -f $RCscript ] ; then 
	echo "Remove: $RCscript"
	rm -f $RCscript
fi
echo "Create: $RCscript"
cat > $RCscript << EOA
#!/bin/sh
#
# Description: Main RC-start SleepTimer script
# Version: $VERSION
#

exec 4>&2 2>$s98err

cat > $daemon << EOF
#!/bin/bash
#
# Description: Daemon for read out of DvdPlayer in named PIPE '/tmp/DvdPlayer.out' and check press any key on RC
# Version: $VERSION
#

####################################
####################################
## Local variables initialization
##
dbgD=$debug							# Debug for sleepD.sh is active or not
####################################
####################################
dbgD_log_path="$debugDLogPath"	# Debug log path
DPout="$DPlog"		# Path of named PIPE for out of DvdPlayer
RCout="$RClog"		# Path of timestamp log of last press key on RC
####################################
####################################
## The main check cycle
##
if [ \\\$dbgD -ge 1 ] ; then echo "Version: $VERSION" > \\\$dbgD_log_path ; fi
while true ; do
	read -r line < \\\$DPout
	#echo ">>\\\$line" >> \\\$dbgD_log_path
	if echo "\\\$line" | grep -v -q "61" ; then
		if [ \\\$dbgD -ge 1 ] ; then echo "\\\$line" >> \\\$dbgD_log_path ; fi
		date '+%s' > \\\$RCout
	fi
	#sleep 1
done
EOF

cat > $script << EOF
#!/bin/bash
#
# Description: Main check script of SleepTimer
# Version: $VERSION
#

####################################
####################################
## Local variables initialization
##
dbgT=$debug													# Debug for sleepT.sh is active or not
time=$latency													# Latency timer shutdown, minutes
####################################
####################################
RCout="$RClog"								# Timestamp log of last press key on RC
if [ ! -e \\\$RCout ] ; then date '+%s' > \\\$RCout ; fi		# First initiated of timestamp log
dbgT_log_path="$debugTLogPath"							# Debug log path

# File for checking of changes
file_01="/tmp/appContext"
	
####################################
####################################
## Function for debugging
##
if [ -f \\\$dbgT_log_path ] ; then 
	rm -f \\\$dbgT_log_path
fi
logmsg (){ 
	if [ \\\$dbgT -ge 1 ]; then   
		D=\\\`date +"%F %T"\\\` 
		echo "[\\\$D] \\\$@" 
		echo "[\\\$D] \\\$@" >> \\\$dbgT_log_path
	fi 
} 

sleep 10

## First start line = 40(122)!!!!!!!!!!
logmsg "40 First start sleepTimer ($VERSION)"
while true ; do
	
	####################################
	####################################
	## First start timer daemon
	##
	logmsg "47 Daemon initialization"						
	i=0
					
	check1_old=\\\`md5sum \\\$file_01 | awk '{ print \\\$1}'\\\`
	check2_old=\\\`md5sum \\\$RCout | awk '{ print \\\$1}'\\\`
	
	until [ \\\$i -eq \\\$time ]
	do
		####################################
		####################################
		## The main check cycle
		##
		
		logmsg "60 Timer = \\\$i"
		logmsg "61 Waiting..."
							
		brk=0
		sleep 60
			
		####################################
		####################################
		## What is being done player: playback Video/Audio/Picture/IPTV, navigation menu, copying files or system idle?
		## Check changes the files: /tmp/appContext, /tmp/PressKey.log
		##
		logmsg "71 Check changes in files"
			
		check1_new=\\\`md5sum \\\$file_01 | awk '{ print \\\$1}'\\\`
		if [ "\\\$check1_new" = "\\\$check1_old" ] ; then
			####################################
			####################################
			## File /tmp/appContext has not changed!
			##
			logmsg "79 /tmp/appContext = not changed"				
					
			####################################
			####################################
			## Check the content of file /tmp/appContext
			##
			systemstate=\\\`grep video_playback \\\$file_01 | awk '{ print \\\$1 }'\\\`
			#logmsg "86 \\\$systemstate"
			if [ "\\\$systemstate" = "video_playback" ] ; then
				####################################
				####################################
				## Player set video playback  - reset timer
				##
				logmsg "92 Playback video -> reset timer"
				i=0
				check1_old=\\\$check1_new
				continue
			fi
					
			systemstate=\\\`grep music_playback \\\$file_01 | awk '{ print \\\$1 }'\\\`
			#logmsg "99 \\\$systemstate"
			if [ "\\\$systemstate" = "music_playback" ] ; then
				####################################
				####################################
				## Player set music playback - reset timer
				##
				logmsg "105 Music playback -> reset timer"
				i=0
				check1_old=\\\$check1_new
				continue
			fi
			
			systemstate=\\\`grep picture_playback \\\$file_01 | awk '{ print \\\$1 }'\\\`
			#logmsg "112 \\\$systemstate"
			if [ "\\\$systemstate" = "picture_playback" ] ; then
				####################################
				####################################
				## Player set picture playback - reset timer
				##
				logmsg "118 Picture playback -> reset timer"
				i=0
				check1_old=\\\$check1_new
				continue
			fi
					
			systemstate=\\\`grep IMSAP \\\$file_01 | awk '{ print \\\$1 }'\\\`
			#logmsg "125 \\\$systemstate"							
			if [ "\\\$systemstate" = "IMSAP" ] ; then
				systemstate=\\\`grep video \\\$file_01 | awk '{ print \\\$1 }'\\\`
				#logmsg "128 \\\$systemstate"
				if [ "\\\$systemstate" = "video" ] ; then
					####################################
					####################################
					## Player IPTV playback - reset timer 
					##
					logmsg "134 IPTV playback -> reset timer"
					i=0
					check1_old=\\\$check1_new
					continue
				fi
							
				systemstate=\\\`grep normal  \\\$file_01 | awk '{ print \\\$1 }'\\\`
				#logmsg "141 \\\$systemstate"							
				if [ "\\\$systemstate" = "normal" ] ; then
					####################################
					####################################
					## Player in navigation menu, copying files or system idle
					##
					check2_new=\\\`md5sum \\\$RCout | awk '{ print \\\$1}'\\\`
					if [ "\\\$check2_new" = "\\\$check2_old" ] ; then
						####################################
						####################################
						## Player in idle - increase the timer
						##
						logmsg "153 System idle -> increase timer"
						i=\\\`expr \\\$i + 1\\\`
					else
						####################################
						####################################
						## Player in navigation menu - reset timer 
						##
						logmsg "160 /tmp/PressKey.log = changed"
						logmsg "161 Menu navigation -> reset timer"
						i=0
						check2_old=\\\$check2_new
						continue
					fi
					
				fi
			fi						
			
			systemstate=\\\`grep unknown \\\$file_01 | awk '{ print \\\$1 }'\\\`
			#logmsg "171 \\\$systemstate"
			if [ "\\\$systemstate" = "unknown" ] ; then
				####################################
				####################################
				## Player system idle after playback Video/Audio/Picture or system idle
				##
				check2_new=\\\`md5sum \\\$RCout | awk '{ print \\\$1}'\\\`
				if [ "\\\$check2_new" = "\\\$check2_old" ] ; then
					####################################
					####################################
					## Player in idle after playback - increase the timer
					##
					logmsg "183 System idle after playback -> increase timer"
					i=\\\`expr \\\$i + 1\\\`
				else
					####################################
					####################################
					## Player in navigation menu - reset timer 
					##
					logmsg "190 /tmp/PressKey.log = changed"
					logmsg "191 Menu navigation -> reset timer"
					i=0
					check2_old=\\\$check2_new
					continue
				fi
			fi
		else
			####################################
			####################################
			## File /tmp/appContext has changed - reset timer
			##
			logmsg "202 /tmp/appContext = changed -> reset timer"
			i=0
			check1_old=\\\$check1_new
			continue
		fi
	done
	####################################
	####################################
	## Power Off......
	##
	logmsg "212 Power Off..."
	#Wi-Fi off and unload driver
	if lsmod | grep -Eq '^8192eu ' ; then
		/usr/bin/killall wpa_supplicant
		/sbin/ifconfig wlan0 down
		/sbin/rmmod 8192eu
	fi
	sleep 1
	echo 0xBC439F00 > /sys/devices/platform/VenusIR/fakekey
	sleep 1
	echo 0xA8579F00 > /sys/devices/platform/VenusIR/fakekey
	sleep 5
	break
done
logmsg "226 Abnormal exit!!!"
EOF
/bin/chmod 755 $daemon
/bin/chmod 755 $script
$daemon&
sleep 1
$script&
sleep 1

exec 2>&4 4>&-
EOA
/bin/chmod 755 $RCscript
####################################
####################################
## Add nessesary strings in /usr/local/etc/rcS.
##
echo "Backup: $rc"
cp -p -f $rc $rc.bak
isReboot=0
if ! grep -q "^mkfifo \$DPout" $rc ; then
	echo "Add: mkfifo \$DPout"
	sed -i -e 's/# This variable tells DvdPlayer the firmware URL for internet updating/mkfifo \$DPout\nsleep 1\n\n&/' $rc
	isReboot=1
fi
if ! grep -q "^DPout=\"\/tmp\/DvdPlayer.out\"; # Path of named PIPE" $rc ; then
	echo "Add: DPout=\"/tmp/DvdPlayer.out\"; # Path of named PIPE"
	sed -i -e 's/mkfifo \$DPout/# Create named PIPE for out of DvdPlayer\nDPout=\"\/tmp\/DvdPlayer.out\"; # Path of named PIPE\n&/' $rc
	isReboot=1
fi
if ! grep -qE '^(([^#]*)(\.\/RootApp DvdPlayer -l on -o \$DPout).*)$' $rc ; then
	echo "Add: ./RootApp DvdPlayer -l on -o \$DPout&"
	sed -ri 's/^[ \t]*([^#]*.*)(\.\/RootApp DvdPlayer)(.*&)$/\t\t#\1\2\3\n\t\t\1\2 -l on -o \$DPout\3/'  $rc
	isReboot=1
fi
if ! grep -qE '^(([^#]*)(\.\/DvdPlayer -l on -o \$DPout).*)$' $rc ; then
	echo "Add: ./DvdPlayer -l on -o \$DPout&"
	sed -ri 's/^[ \t]*([^#]*.*)(\.\/DvdPlayer)(.*&)$/\t\t#\1\2\3\n\t\t\1\2 -l on -o \$DPout\3/'  $rc
	isReboot=1
fi
####################################
####################################
## Start main RC-start SleepTimer script or reboot system, if necessary
##
if [ "$isReboot" -eq "1" ] ; then
	while true; do
		echo -e >&4 "\n"
		echo -e >&4 "For finish install scripts need reboot your player - Y/N?\n"
		read yn
		echo -e >&4 "\n"
		case $yn in
			[Yy]* ) 
				echo >&4 "Reboot............."
				echo "Reboot............."
				/usr/local/etc/mos/bin/reboot.sh&
				break
				;;
			[Nn]* )
				echo "No reboot!"
				break
				;;
			* ) echo >&4 "Please answer Y[es] or N[o]!";;
		esac
	done
else
	if ! ps -aux | grep /tmp/sleep | grep -qv grep; then
		echo "Start: $RCscript"
		$RCscript&
	fi
fi
echo "Exit installer!"
exec 1>&3 2>&4